@@ -5,7 +5,7 @@ use 5.014002;
 use strict;
 use warnings;
 
-use version; our $VERSION = version->declare( "v1.1.25" );
+use version; our $VERSION = version->declare( "v1.1.33" );
 
 ###
 ### This test module implements DNSSEC tests.
@@ -463,52 +463,59 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     ALGORITHM_DEPRECATED => sub {
         __x    # DNSSEC:ALGORITHM_DEPRECATED
-          'The DNSKEY with tag {keytag} uses deprecated algorithm number {algorithm} ({description}).',
+          'The DNSKEY with tag {keytag} uses deprecated algorithm number '
+          . '{algo_num} ({algo_descr}).',
           @_;
     },
     ALGORITHM_NOT_RECOMMENDED => sub {
         __x    # DNSSEC:ALGORITHM_NOT_RECOMMENDED
-          'The DNSKEY with tag {keytag} uses an algorithm number {algorithm} ({description}) '
-          . 'which is not recommended to be used.',
+          'The DNSKEY with tag {keytag} uses an algorithm number '
+          . '{algo_num} ({algo_descr}) which is not recommended to be used.',
           @_;
     },
     ALGORITHM_NOT_ZONE_SIGN => sub {
         __x    # DNSSEC:ALGORITHM_NOT_ZONE_SIGN
-          'The DNSKEY with tag {keytag} uses algorithm number not meant for zone signing, '
-          . 'algorithm number {algorithm} ({description}).',
+          'The DNSKEY with tag {keytag} uses algorithm number not meant for '
+          . 'zone signing, algorithm number {algo_num} ({algo_descr}).',
           @_;
     },
     ALGORITHM_OK => sub {
         __x    # DNSSEC:ALGORITHM_OK
-          'The DNSKEY with tag {keytag} uses algorithm number {algorithm} ({description}), which is OK.',
+          'The DNSKEY with tag {keytag} uses algorithm number {algo_num} '
+          . '({algo_descr}), which is OK.',
           @_;
     },
     ALGORITHM_PRIVATE => sub {
         __x    # DNSSEC:ALGORITHM_PRIVATE
-          'The DNSKEY with tag {keytag} uses private algorithm number {algorithm} ({description}).', @_;
+          'The DNSKEY with tag {keytag} uses private algorithm number '
+          . '{algo_num} ({algo_descr}).',
+          @_;
     },
     ALGORITHM_RESERVED => sub {
         __x    # DNSSEC:ALGORITHM_RESERVED
-          'The DNSKEY with tag {keytag} uses reserved algorithm number {algorithm} ({description}).', @_;
+          'The DNSKEY with tag {keytag} uses reserved algorithm number '
+          . '{algo_num} ({algo_descr}).',
+          @_;
     },
     ALGORITHM_UNASSIGNED => sub {
         __x    # DNSSEC:ALGORITHM_UNASSIGNED
-          'The DNSKEY with tag {keytag} uses unassigned algorithm number {algorithm} ({description}).', @_;
+          'The DNSKEY with tag {keytag} uses unassigned algorithm number '
+          . '{algo_num} ({algo_descr}).',
+          @_;
     },
     ALGO_NOT_SIGNED_RRSET => sub {
         __x    # DNSSEC:ALGO_NOT_SIGNED_RRSET
-          'Nameserver {ns} responded with no RRSIG for RRset {rrtype} created by the '
-          . 'algorithm {algorithm}.',
+          'Nameserver {ns} responded with no RRSIG for RRset {rrtype} created '
+          . 'by the algorithm {algo_num}.',
           @_;
     },
     ALL_ALGO_SIGNED => sub {
         __x    # DNSSEC:ALL_ALGO_SIGNED
-          'All the tested RRset (SOA/DNSKEY/NS) are signed by each algorithm present in the DNSKEY RRset.',
-          @_;
+          'All the tested RRset (SOA/DNSKEY/NS) are signed by each algorithm present in the DNSKEY RRset.', @_;
     },
     BROKEN_DNSSEC => sub {
         __x    # DNSSEC:BROKEN_DNSSEC
-          'All nameservers for zone {zone} responds with neither NSEC nor NSEC3 records when such '
+          'All nameservers for zone {domain} responds with neither NSEC nor NSEC3 records when such '
           . 'records are expected.',
           @_;
     },
@@ -559,11 +566,11 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     DNSKEY_SIGNATURE_NOT_OK => sub {
         __x    # DNSSEC:DNSKEY_SIGNATURE_NOT_OK
-          'Signature for DNSKEY with tag {signature} failed to verify with error \'{error}\'.', @_;
+          'Signature for DNSKEY with tag {keytag} failed to verify with error \'{error}\'.', @_;
     },
     DNSKEY_SIGNATURE_OK => sub {
         __x    # DNSSEC:DNSKEY_SIGNATURE_OK
-          'A signature for DNSKEY with tag {signature} was correctly signed.', @_;
+          'A signature for DNSKEY with tag {keytag} was correctly signed.', @_;
     },
     DNSKEY_SIGNED => sub {
         __x    # DNSSEC:DNSKEY_SIGNED
@@ -571,61 +578,65 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     DNSKEY_SMALLER_THAN_REC => sub {
         __x    # DNSSEC:DNSKEY_SMALLER_THAN_REC
-          'DNSKEY with tag {keytag} and using algorithm {algorithm_number} ({algorithm_description}) '
-          . 'has a size ({keysize}) smaller than the recommended one ({keysizerec}).',
+          'DNSKEY with tag {keytag} and using algorithm {algo_num} '
+          . '({algo_descr}) has a size ({keysize}) smaller than the '
+          . 'recommended one ({keysizerec}).',
           @_;
     },
     DNSKEY_TOO_SMALL_FOR_ALGO => sub {
         __x    # DNSSEC:DNSKEY_TOO_SMALL_FOR_ALGO
-          'DNSKEY with tag {keytag} and using algorithm {algorithm_number} ({algorithm_description}) '
-          . 'has a size ({keysize}) smaller than the minimum one ({keysizemin}).',
+          'DNSKEY with tag {keytag} and using algorithm {algo_num} '
+          . '({algo_descr}) has a size ({keysize}) smaller than the minimum '
+          . 'one ({keysizemin}).',
           @_;
     },
     DNSKEY_TOO_LARGE_FOR_ALGO => sub {
         __x    # DNSSEC:DNSKEY_TOO_LARGE_FOR_ALGO
-          'DNSKEY with tag {keytag} and using algorithm {algorithm_number} ({algorithm_description}) '
-          . 'has a size ({keysize}) larger than the maximum one ({keysizemax}).',
+          'DNSKEY with tag {keytag} and using algorithm {algo_num} '
+          . '({algo_descr}) has a size ({keysize}) larger than the maximum one '
+          . '({keysizemax}).',
           @_;
     },
     DS_ALGORITHM_NOT_DS => sub {
         __x    # DNSSEC:DS_ALGORITHM_NOT_DS
-          '{ns} returned a DS record created by algorithm {algorithm_number} '
-          . '({algorithm_mnemonic}) which is not meant for DS. The DS record is for the DNSKEY '
-          . 'record with keytag {keytag} in zone {zone}.',
+          '{ns} returned a DS record created by algorithm {algo_num} '
+          . '({algo_mnemo}) which is not meant for DS. The DS record is for '
+          . 'the DNSKEY record with keytag {keytag} in zone {domain}.',
           @_;
     },
     DS_ALGORITHM_DEPRECATED => sub {
         __x    # DNSSEC:DS_ALGORITHM_DEPRECATED
-          '{ns} returned a DS record created by algorithm {algorithm_number} '
-          . '({algorithm_mnemonic}), which is deprecated. The DS record is for the DNSKEY '
-          . 'record with keytag {keytag} in zone {zone}.',
+          '{ns} returned a DS record created by algorithm {algo_num} '
+          . '({algo_mnemo}), which is deprecated. The DS record is for the '
+          . 'DNSKEY record with keytag {keytag} in zone {domain}.',
           @_;
     },
     DS_ALGORITHM_MISSING => sub {
         __x    # DNSSEC:DS_ALGORITHM_MISSING
-          '{ns} returned no DS record created by algorithm {algorithm_number} '
-          . '({algorithm_mnemonic}) for zone {zone}, which is required.',
+          '{ns} returned no DS record created by algorithm {algo_num} '
+          . '({algo_mnemo}) for zone {domain}, which is required.',
           @_;
     },
     DS_ALGORITHM_OK => sub {
         __x    # DNSSEC:DS_ALGORITHM_OK
-          '{ns} returned a DS record created by algorithm {algorithm_number} '
-          . '({algorithm_mnemonic}), which is OK. The DS record is for the DNSKEY record with '
-          . 'keytag {keytag} in zone {zone}.',
+          '{ns} returned a DS record created by algorithm {algo_num} '
+          . '({algo_mnemo}), which is OK. The DS record is for the DNSKEY '
+          . 'record with keytag {keytag} in zone {domain}.',
           @_;
     },
     DS_ALGORITHM_RESERVED => sub {
         __x    # DNSSEC:DS_ALGORITHM_RESERVED
-          '{ns} returned a DS record created by with an algorithm not assigned (algorithm number '
-          . '{algorithm_number}), which is not OK. The DS record is for the DNSKEY record with keytag {keytag} '
-          . 'in zone {zone}.',
+          '{ns} returned a DS record created by with an algorithm not assigned '
+          . '(algorithm number {algo_num}), which is not OK. The DS record is '
+          . 'for the DNSKEY record with keytag {keytag} in zone {domain}.',
           @_;
     },
     DS_ALGO_SHA1_DEPRECATED => sub {
         __x    # DNSSEC:DS_ALGO_SHA1_DEPRECATED
-          'Nameserver {ns} returned a DS record created by algorithm {algorithm_number} '
-          . '({algorithm_mnemonic}) which is deprecated, while it is still widely used. The DS record is '
-          . 'for the DNSKEY record with keytag {keytag} in zone {zone}.',
+          'Nameserver {ns} returned a DS record created by algorithm '
+          . '{algo_num} ({algo_mnemo}) which is deprecated, while it is still '
+          . 'widely used. The DS record is for the DNSKEY record with keytag '
+          . '{keytag} in zone {domain}.',
           @_;
     },
     DS_BUT_NOT_DNSKEY => sub {
@@ -634,17 +645,17 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     DS_MATCHES => sub {
         __x    # DNSSEC:DS_MATCHES
-          'The DS records in the parent zone match DNSKEY records in the child zone.',  @_;
+          'The DS records in the parent zone match DNSKEY records in the child zone.', @_;
     },
     DURATION_LONG => sub {
         __x    # DNSSEC:DURATION_LONG
-          'RRSIG with keytag {tag} and covering type(s) {types} '
+          'RRSIG with keytag {keytag} and covering type(s) {types} '
           . 'has a duration of {duration} seconds, which is too long.',
           @_;
     },
     DURATION_OK => sub {
         __x    # DNSSEC:DURATION_OK
-          'RRSIG with keytag {tag} and covering type(s) {types} '
+          'RRSIG with keytag {keytag} and covering type(s) {types} '
           . 'has a duration of {duration} seconds, which is just fine.',
           @_;
     },
@@ -666,20 +677,16 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     INCONSISTENT_DNSSEC => sub {
         __x    # DNSSEC:INCONSISTENT_DNSSEC
-          'Some, but not all, nameservers for zone {zone} respond with neither NSEC nor NSEC3 records when '
+          'Some, but not all, nameservers for zone {domain} respond with neither NSEC nor NSEC3 records when '
           . 'such records are expected.',
           @_;
     },
     INCONSISTENT_NSEC_NSEC3 => sub {
         __x    # DNSSEC:INCONSISTENT_NSEC_NSEC3
-          'Some nameservers for zone {zone} respond with NSEC records and others respond with NSEC3 records. '
+          'Some nameservers for zone {domain} respond with NSEC records and others respond with NSEC3 records. '
           . 'Consistency is expected.',
           @_;
     },
-    INVALID_NAME_RCODE => sub {
-        __x    # DNSSEC:INVALID_NAME_RCODE
-          'When asked for the name {name}, which must not exist, the response had RCODE {rcode}.', @_;
-    },
     IPV4_DISABLED => sub {
         __x    # DNSSEC:IPV4_DISABLED
           'IPv4 is disabled, not sending "{rrtype}" query to {ns}.', @_;
@@ -697,16 +704,16 @@ Readonly my %TAG_DESCRIPTIONS => (
           'Key with keytag {keytag} details : Size = {keysize}, Flags ({sep}, {rfc5011}).', @_;
     },
     KEY_SIZE_OK => sub {
-          __x    # DNSSEC:KEY_SIZE_OK
-            'All keys from the DNSKEY RRset have the correct size.', @_;
-      },
+        __x    # DNSSEC:KEY_SIZE_OK
+          'All keys from the DNSKEY RRset have the correct size.', @_;
+    },
     MANY_ITERATIONS => sub {
         __x    # DNSSEC:MANY_ITERATIONS
           'The number of NSEC3 iterations is {count}, which is on the high side.', @_;
     },
     MIXED_NSEC_NSEC3 => sub {
         __x    # DNSSEC:MIXED_NSEC_NSEC3
-          'Nameserver {ns} for zone {zone} responds with both NSEC and NSEC3 '
+          'Nameserver {ns} for zone {domain} responds with both NSEC and NSEC3 '
           . 'records when only one record type is expected.',
           @_;
     },
@@ -720,8 +727,7 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     NO_KEYS_OR_NO_SIGS => sub {
         __x    # DNSSEC:NO_KEYS_OR_NO_SIGS
-          'Cannot test DNSKEY signatures, because we got {keys} DNSKEY records and {sigs} RRSIG records.',
-          @_;
+          'Cannot test DNSKEY signatures, because we got {keys} DNSKEY records and {sigs} RRSIG records.', @_;
     },
     NO_KEYS_OR_NO_SIGS_OR_NO_SOA => sub {
         __x    # DNSSEC:NO_KEYS_OR_NO_SIGS_OR_NO_SOA
@@ -748,8 +754,8 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     NO_NSEC_NSEC3 => sub {
         __x    # DNSSEC:NO_NSEC_NSEC3
-          'Nameserver {ns} for zone {zone} responds with neither NSEC nor NSEC3 record when '
-          . 'when such records are expected.',
+          'Nameserver {ns} for zone {domain} responds with neither NSEC nor '
+          . 'NSEC3 record when when such records are expected.',
           @_;
     },
     NO_RESPONSE_DNSKEY => sub {
@@ -758,7 +764,7 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     NO_RESPONSE_DS => sub {
         __x    # DNSSEC:NO_RESPONSE_DS
-          '{ns} returned no DS records for {zone}.', @_;
+          '{ns} returned no DS records for {domain}.', @_;
     },
     NO_RESPONSE_RRSET => sub {
         __x    # DNSSEC:NO_RESPONSE_RRSET
@@ -778,7 +784,7 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     NSEC3_COVERS_NOT => sub {
         __x    # DNSSEC:NSEC3_COVERS_NOT
-          'NSEC3 record does not cover {name}.', @_;
+          'NSEC3 record does not cover {domain}.', @_;
     },
     NSEC3_NOT_SIGNED => sub {
         __x    # DNSSEC:NSEC3_NOT_SIGNED
@@ -786,11 +792,11 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     NSEC3_SIG_VERIFY_ERROR => sub {
         __x    # DNSSEC:NSEC3_SIG_VERIFY_ERROR
-          'Trying to verify NSEC3 RRset with RRSIG {sig} gave error \'{error}\'.', @_;
+          'Trying to verify NSEC3 RRset with RRSIG {keytag} gave error \'{error}\'.', @_;
     },
     NSEC_COVERS_NOT => sub {
         __x    # DNSSEC:NSEC_COVERS_NOT
-          'NSEC does not cover {name}.', @_;
+          'NSEC does not cover {domain}.', @_;
     },
     NSEC_NOT_SIGNED => sub {
         __x    # DNSSEC:NSEC_NOT_SIGNED
@@ -798,23 +804,23 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     NSEC_SIG_VERIFY_ERROR => sub {
         __x    # DNSSEC:NSEC_SIG_VERIFY_ERROR
-          'Trying to verify NSEC RRset with RRSIG {sig} gave error \'{error}\'.', @_;
+          'Trying to verify NSEC RRset with RRSIG {keytag} gave error \'{error}\'.', @_;
     },
     REMAINING_LONG => sub {
         __x    # DNSSEC:REMAINING_LONG
-          'RRSIG with keytag {tag} and covering type(s) {types} '
+          'RRSIG with keytag {keytag} and covering type(s) {types} '
           . 'has a remaining validity of {duration} seconds, which is too long.',
           @_;
     },
     REMAINING_SHORT => sub {
         __x    # DNSSEC:REMAINING_SHORT
-          'RRSIG with keytag {tag} and covering type(s) {types} '
+          'RRSIG with keytag {keytag} and covering type(s) {types} '
           . 'has a remaining validity of {duration} seconds, which is too short.',
           @_;
     },
     RRSIG_EXPIRATION => sub {
         __x    # DNSSEC:RRSIG_EXPIRATION
-          'RRSIG with keytag {tag} and covering type(s) {types} expires at : {date}.', @_;
+          'RRSIG with keytag {keytag} and covering type(s) {types} expires at ' . ': {date}.', @_;
     },
     RRSET_NOT_SIGNED => sub {
         __x    # DNSSEC:RRSET_NOT_SIGNED
@@ -828,8 +834,8 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     RRSIG_EXPIRED => sub {
         __x    # DNSSEC:RRSIG_EXPIRED
-          'RRSIG with keytag {tag} and covering type(s) {types} has already expired (expiration '
-          . 'is: {expiration}).',
+          'RRSIG with keytag {keytag} and covering type(s) {types} has already '
+          . 'expired (expiration is: {expiration}).',
           @_;
     },
     RRSIG_NOT_MATCH_DNSKEY => sub {
@@ -842,11 +848,11 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     SOA_SIGNATURE_NOT_OK => sub {
         __x    # DNSSEC:SOA_SIGNATURE_NOT_OK
-          'Trying to verify SOA RRset with signature {signature} gave error \'{error}\'.', @_;
+          'Trying to verify SOA RRset with signature {keytag} gave error \'{error}\'.', @_;
     },
     SOA_SIGNATURE_OK => sub {
         __x    # DNSSEC:SOA_SIGNATURE_OK
-          'RRSIG {signature} correctly signs SOA RRset.', @_;
+          'RRSIG {keytag} correctly signs SOA RRset.', @_;
     },
     SOA_SIGNED => sub {
         __x    # DNSSEC:SOA_SIGNED
@@ -854,7 +860,7 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     TEST_ABORTED => sub {
         __x    # DNSSEC:TEST_ABORTED
-          'Nameserver {ns} for zone {zone} responds with RCODE "NOERROR" on a query that '
+          'Nameserver {ns} for zone {domain} responds with RCODE "NOERROR" on a query that '
           . 'is expected to give response with RCODE "NXDOMAIN". Test for NSEC and NSEC3 is aborted '
           . 'for this nameserver.',
           @_;
@@ -873,7 +879,7 @@ Readonly my %TAG_DESCRIPTIONS => (
     },
     UNEXPECTED_RESPONSE_DS => sub {
         __x    # DNSSEC:UNEXPECTED_RESPONSE_DS
-          'Nameserver {ns} responded with an unexpected rcode ({rcode}) on a DS query for zone {zone}.', @_;
+          'Nameserver {ns} responded with an unexpected rcode ({rcode}) on a DS query for zone {domain}.', @_;
     },
 );
 
@@ -895,62 +901,128 @@ sub dnssec01 {
 
     if ( my $parent = $zone->parent ) {
         foreach my $ns ( @{ $parent->ns } ) {
-            my $ns_args = {
-                ns     => $ns->string,
-                zone   => q{} . $zone->name,
-                rrtype => q{DS},
-            };
 
             if ( not Zonemaster::Engine::Profile->effective->get(q{net.ipv6}) and $ns->address->version == $IP_VERSION_6 ) {
-                push @results, info( IPV6_DISABLED => $ns_args );
+                push @results,
+                  info(
+                    IPV6_DISABLED => {
+                        ns     => $ns->string,
+                        rrtype => q{DS},
+                    }
+                  );
                 next;
             }
 
             if ( not Zonemaster::Engine::Profile->effective->get(q{net.ipv4}) and $ns->address->version == $IP_VERSION_4 ) {
-                push @results, info( IPV4_DISABLED => $ns_args );
+                push @results,
+                  info(
+                    IPV4_DISABLED => {
+                        ns     => $ns->string,
+                        rrtype => q{DS},
+                    }
+                  );
                 next;
             }
 
             my $ds_p = $ns->query( $zone->name, q{DS}, { usevc => 0, dnssec => 1 } );
             if ( not $ds_p ) {
-                push @results, info( NO_RESPONSE_DS => $ns_args );
+                push @results,
+                  info(
+                    NO_RESPONSE_DS => {
+                        ns     => $ns->string,
+                        domain => q{} . $zone->name,
+                    }
+                  );
                 next;
             }
             elsif ($ds_p->rcode ne q{NOERROR} ) {
-                $ns_args->{rcode} = $ds_p->rcode;
-                push @results, info( UNEXPECTED_RESPONSE_DS => $ns_args );
+                push @results,
+                  info(
+                    UNEXPECTED_RESPONSE_DS => {
+                        ns     => $ns->string,
+                        domain => q{} . $zone->name,
+                        rcode  => $ds_p->rcode,
+                    }
+                  );
                 next;
             }
             else {
                 my $algorithm2 = 0;
                 my @dss = $ds_p->get_records( q{DS}, q{answer} );
-                foreach my $ds ( @dss ) {
-                    my $ds_args = { %$ns_args };
-                    $ds_args->{keytag} = $ds->keytag;
-                    $ds_args->{algorithm_number} = $ds->digtype;
-                    $ds_args->{algorithm_mnemonic} = $digest_algorithms{$ds->digtype};
+                foreach my $ds (@dss) {
+                    my $mnemonic = $digest_algorithms{ $ds->digtype };
                     if ( $ds->digtype == 0 ) {
-                        push @results, info( DS_ALGORITHM_NOT_DS => $ds_args );
+                        push @results,
+                          info(
+                            DS_ALGORITHM_NOT_DS => {
+                                ns         => $ns->string,
+                                domain     => q{} . $zone->name,
+                                keytag     => $ds->keytag,
+                                algo_num   => $ds->digtype,
+                                algo_mnemo => $mnemonic,
+                            }
+                          );
                     }
                     elsif ( $ds->digtype == 1 ) {
-                        push @results, info( DS_ALGO_SHA1_DEPRECATED => $ds_args );
+                        push @results,
+                          info(
+                            DS_ALGO_SHA1_DEPRECATED => {
+                                ns         => $ns->string,
+                                domain     => q{} . $zone->name,
+                                keytag     => $ds->keytag,
+                                algo_num   => $ds->digtype,
+                                algo_mnemo => $mnemonic,
+                            }
+                          );
                     }
                     elsif ( $ds->digtype == 3 ) {
-                        push @results, info( DS_ALGORITHM_DEPRECATED => $ds_args );
+                        push @results,
+                          info(
+                            DS_ALGORITHM_DEPRECATED => {
+                                ns         => $ns->string,
+                                domain     => q{} . $zone->name,
+                                keytag     => $ds->keytag,
+                                algo_num   => $ds->digtype,
+                                algo_mnemo => $mnemonic,
+                            }
+                          );
                     }
                     elsif ( $ds->digtype >= 5 and $ds->digtype <= 255 ) {
-                        push @results, info( DS_ALGORITHM_RESERVED => $ds_args );
+                        push @results,
+                          info(
+                            DS_ALGORITHM_RESERVED => {
+                                ns         => $ns->string,
+                                domain     => q{} . $zone->name,
+                                keytag     => $ds->keytag,
+                                algo_num   => $ds->digtype,
+                                algo_mnemo => $mnemonic,
+                            }
+                          );
                     }
                     else {
                         $algorithm2++ if $ds->digtype == 2;
-                        push @results, info( DS_ALGORITHM_OK => $ds_args );
+                        push @results,
+                          info(
+                            DS_ALGORITHM_OK => {
+                                ns         => $ns->string,
+                                domain     => q{} . $zone->name,
+                                keytag     => $ds->keytag,
+                                algo_num   => $ds->digtype,
+                                algo_mnemo => $mnemonic,
+                            }
+                          );
                     }
                 }
                 if ( not $algorithm2 ) {
-                    my $ds_args = { %$ns_args };
-                    $ds_args->{algorithm_number} = 2;
-                    $ds_args->{algorithm_mnemonic} = $digest_algorithms{2};
-                    push @results, info( DS_ALGORITHM_MISSING => $ds_args );
+                    push @results,
+                      info(
+                        DS_ALGORITHM_MISSING => {
+                            ns         => $ns->string,
+                            domain     => q{} . $zone->name,
+                            algo_num   => 2,
+                            algo_mnemo => $digest_algorithms{2},
+                        }
+                      );
                 }
             }    
         }
@@ -989,8 +1061,14 @@ sub dnssec02 {
                 next;
             }
             elsif ($ds_p->rcode ne q{NOERROR} ) {
-                $ns_args->{rcode} = $ds_p->rcode;
-                push @results, info( UNEXPECTED_RESPONSE_DS => $ns_args );
+                push @results,
+                  info(
+                    UNEXPECTED_RESPONSE_DS => {
+                        ns     => $ns->string,
+                        domain => q{} . $zone->name,
+                        rcode  => $ds_p->rcode,
+                    }
+                  );
                 next;
             }
             else {
@@ -1226,37 +1304,41 @@ sub dnssec04 {
         push @results,
           info(
             RRSIG_EXPIRATION => {
-                date  => scalar( gmtime($sig->expiration) ),
-                tag   => $sig->keytag,
-                types => $sig->typecovered,
+                date   => scalar( gmtime($sig->expiration) ),
+                keytag => $sig->keytag,
+                types  => $sig->typecovered,
             }
           );
 
         my $remaining = $sig->expiration - int( $dnskey_p->timestamp );
         my $result_remaining;
+        my $remaining_short_limit = Zonemaster::Engine::Profile->effective->get( q{test_cases_vars.dnssec04.REMAINING_SHORT} );
+        my $remaining_long_limit  = Zonemaster::Engine::Profile->effective->get( q{test_cases_vars.dnssec04.REMAINING_LONG} );
+        my $duration_long_limit   = Zonemaster::Engine::Profile->effective->get( q{test_cases_vars.dnssec04.DURATION_LONG} );
+
         if ( $remaining < 0 ) {    # already expired
             $result_remaining = info(
                 RRSIG_EXPIRED => {
                     expiration => $sig->expiration,
-                    tag        => $sig->keytag,
+                    keytag     => $sig->keytag,
                     types      => $sig->typecovered,
                 }
             );
         }
-        elsif ( $remaining < ( $DURATION_12_HOURS_IN_SECONDS ) ) {
+        elsif ( $remaining < ( $remaining_short_limit ) ) {
             $result_remaining = info(
                 REMAINING_SHORT => {
                     duration => $remaining,
-                    tag      => $sig->keytag,
+                    keytag   => $sig->keytag,
                     types    => $sig->typecovered,
                 }
             );
         }
-        elsif ( $remaining > ( $DURATION_180_DAYS_IN_SECONDS ) ) {
+        elsif ( $remaining > ( $remaining_long_limit ) ) {
             $result_remaining = info(
                 REMAINING_LONG => {
                     duration => $remaining,
-                    tag      => $sig->keytag,
+                    keytag   => $sig->keytag,
                     types    => $sig->typecovered,
                 }
             );
@@ -1264,11 +1346,11 @@ sub dnssec04 {
 
         my $duration = $sig->expiration - $sig->inception;
         my $result_duration;
-        if ( $duration > ( $DURATION_180_DAYS_IN_SECONDS ) ) {
+        if ( $duration > ( $duration_long_limit ) ) {
             $result_duration = info(
                 DURATION_LONG => {
                     duration => $duration,
-                    tag      => $sig->keytag,
+                    keytag   => $sig->keytag,
                     types    => $sig->typecovered,
                 }
             );
@@ -1283,7 +1365,7 @@ sub dnssec04 {
               info(
                 DURATION_OK => {
                     duration => $duration,
-                    tag      => $sig->keytag,
+                    keytag   => $sig->keytag,
                     types    => $sig->typecovered,
                 }
               );
@@ -1341,9 +1423,9 @@ sub dnssec05 {
         foreach my $key ( @keys ) {
             my $algo      = $key->algorithm;
             my $algo_args = {
-                algorithm   => $algo,
+                algo_num    => $algo,
                 keytag      => $key->keytag,
-                description => $algo_properties{$algo}{description},
+                algo_descr  => $algo_properties{$algo}{description},
             };
 
             if ( $algo_properties{$algo}{status} == $ALGO_STATUS_DEPRECATED ) {
@@ -1503,7 +1585,7 @@ sub dnssec08 {
         return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
     }
 
-    my $ok = 0;
+    my $ok = undef;
     foreach my $sig ( @sigs ) {
         my $msg  = q{};
         my $time = $dnskey_p->timestamp;
@@ -1511,7 +1593,7 @@ sub dnssec08 {
             push @results,
               info(
                 DNSKEY_SIGNATURE_OK => {
-                    signature => $sig->keytag,
+                    keytag => $sig->keytag,
                 }
               );
             $ok = $sig->keytag;
@@ -1523,15 +1605,15 @@ sub dnssec08 {
             push @results,
               info(
                 DNSKEY_SIGNATURE_NOT_OK => {
-                    signature => $sig->keytag,
-                    error     => $msg,
-                    time      => $time,
+                    keytag => $sig->keytag,
+                    error  => $msg,
+                    time   => $time,
                 }
               );
         }
     } ## end foreach my $sig ( @sigs )
 
-    if ( $ok ) {
+    if ( defined $ok ) {
         push @results,
           info(
             DNSKEY_SIGNED => {
@@ -1576,7 +1658,7 @@ sub dnssec09 {
         return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
     }
 
-    my $ok = 0;
+    my $ok = undef;
     foreach my $sig ( @sigs ) {
         my $msg  = q{};
         my $time = $soa_p->timestamp;
@@ -1584,7 +1666,7 @@ sub dnssec09 {
             push @results,
               info(
                 SOA_SIGNATURE_OK => {
-                    signature => $sig->keytag,
+                    keytag => $sig->keytag,
                 }
               );
             $ok = $sig->keytag;
@@ -1596,14 +1678,14 @@ sub dnssec09 {
             push @results,
               info(
                 SOA_SIGNATURE_NOT_OK => {
-                    signature => $sig->keytag,
-                    error     => $msg,
+                    keytag => $sig->keytag,
+                    error  => $msg,
                 }
               );
         }
     } ## end foreach my $sig ( @sigs )
 
-    if ( $ok ) {
+    if ( defined $ok ) {
         push @results,
           info(
             SOA_SIGNED => {
@@ -1662,8 +1744,8 @@ sub dnssec10 {
             push @results,
               info(
                 TEST_ABORTED => {
-                    ns   => $ns->string,
-                    zone => $zone->name->string,
+                    ns     => $ns->string,
+                    domain => $zone->name->string,
                 }
               );
         }
@@ -1678,18 +1760,22 @@ sub dnssec10 {
             my @nsec  = $a_p->get_records( q{NSEC}, q{authority} );
             my @nsec3 = $a_p->get_records( q{NSEC3}, q{authority} );
             if ( scalar @nsec and scalar @nsec3 ) {
-                my $args = {
-                    ns   => $ns->string,
-                    zone => $zone->name->string,
-                };
-                push @results, info( MIXED_NSEC_NSEC3 => $args );
+                push @results,
+                  info(
+                    MIXED_NSEC_NSEC3 => {
+                        ns     => $ns->string,
+                        domain => $zone->name->string,
+                    }
+                  );
             }
             elsif ( not scalar @nsec and not scalar @nsec3 ) {
-                my $args = {
-                    ns   => $ns->string,
-                    zone => $zone->name->string,
-                };
-                push @results, info( NO_NSEC_NSEC3 => $args );
+                push @results,
+                  info(
+                    NO_NSEC_NSEC3 => {
+                        ns     => $ns->string,
+                        domain => $zone->name->string,
+                    }
+                  );
                 $no_dnssec_zone{$ns->address->short}++;
             }
             elsif ( scalar @nsec and not scalar @nsec3 ) {
@@ -1709,7 +1795,7 @@ sub dnssec10 {
                             foreach my $sig ( @sigs ) {
                                 my $msg = q{};
                                 if ( not scalar @dnskeys ) {
-                                    push @results, info( NSEC_SIG_VERIFY_ERROR => { error => q{DNSKEY missing}, sig => $sig->keytag } );
+                                    push @results, info( NSEC_SIG_VERIFY_ERROR => { error => q{DNSKEY missing}, keytag => $sig->keytag } );
                                 }
                                 elsif (
                                     $sig->verify_time(
@@ -1727,8 +1813,8 @@ sub dnssec10 {
                                     push @results,
                                       info(
                                         NSEC_SIG_VERIFY_ERROR => {
-                                            error => $msg,
-                                            sig   => $sig->keytag,
+                                            error  => $msg,
+                                            keytag => $sig->keytag,
                                         }
                                       );
                                 }
@@ -1740,7 +1826,7 @@ sub dnssec10 {
                     }
                 }
                 if ( not $covered ) {
-                    push @results, info( NSEC_COVERS_NOT => {} );
+                    push @results, info( NSEC_COVERS_NOT => { domain => $non_existent_domain_name } );
                 }
             }
             elsif ( not scalar @nsec and scalar @nsec3 ) {
@@ -1760,7 +1846,7 @@ sub dnssec10 {
                             foreach my $sig ( @sigs ) {
                                 my $msg = q{};
                                 if ( not scalar @dnskeys ) {
-                                    push @results, info( NSEC3_SIG_VERIFY_ERROR => { error => 'DNSKEY missing', sig => $sig->keytag } );
+                                    push @results, info( NSEC3_SIG_VERIFY_ERROR => { error => 'DNSKEY missing', keytag => $sig->keytag } );
                                 }
                                 elsif (
                                     $sig->verify_time(
@@ -1778,8 +1864,8 @@ sub dnssec10 {
                                     push @results,
                                       info(
                                         NSEC3_SIG_VERIFY_ERROR => {
-                                            error => $msg,
-                                            sig   => $sig->keytag,
+                                            error  => $msg,
+                                            keytag => $sig->keytag,
                                         }
                                       );
                                 }
@@ -1791,20 +1877,20 @@ sub dnssec10 {
                     }
                 }
                 if ( not $covered ) {
-                    push @results, info( NSEC3_COVERS_NOT => {} );
+                    push @results, info( NSEC3_COVERS_NOT => { domain => $non_existent_domain_name } );
                 }
             }
         }
     }
 
     if ( scalar keys %no_dnssec_zone and ( scalar keys %nsec_zone or scalar keys %nsec3_zone ) ) {
-        push @results, info( INCONSISTENT_DNSSEC => { zone => $zone->name->string } );
+        push @results, info( INCONSISTENT_DNSSEC => { domain => $zone->name->string } );
     }
     elsif ( scalar keys %no_dnssec_zone and not scalar keys %nsec_zone and not scalar keys %nsec3_zone ) {
-        push @results, info( BROKEN_DNSSEC => { zone => $zone->name->string } );
+        push @results, info( BROKEN_DNSSEC => { domain => $zone->name->string } );
     }
     elsif ( scalar keys %nsec_zone and scalar keys %nsec3_zone ) {
-        push @results, info( INCONSISTENT_NSEC_NSEC3 => { zone => $zone->name->string } );
+        push @results, info( INCONSISTENT_NSEC_NSEC3 => { domain => $zone->name->string } );
     }
     elsif ( scalar keys %nsec_zone and not grep { $_->tag eq q{MIXED_NSEC_NSEC3} } @results ) {
         push @results, info( HAS_NSEC => {} );
@@ -1839,7 +1925,7 @@ sub dnssec11 {
     my %dnskey = map { $_->keytag => $_ } $dnskey_p->get_records_for_name( 'DNSKEY', $zone->name->string );
     my %rrsig  = map { $_->keytag => $_ } $dnskey_p->get_records_for_name( 'RRSIG',  $zone->name->string );
 
-    my $pass = 0;
+    my $pass = undef;
     my @fail;
     if ( scalar( keys %ds ) > 0 ) {
         foreach my $tag ( keys %ds ) {
@@ -1880,7 +1966,7 @@ sub dnssec11 {
         push @fail, 'no_ds';
     }
 
-    if ($pass) {
+    if ( defined $pass ) {
         push @results, info( DELEGATION_SIGNED => { keytag => $pass } )
     } else {
         push @results, info( DELEGATION_NOT_SIGNED => { keytag => 'info', reason => join(';', @fail) } )
@@ -1964,7 +2050,7 @@ sub dnssec13 {
             foreach my $algorithm ( @algorithms ) {
                 if ( not scalar grep { $_->algorithm == $algorithm } @sigs ) {
                     $all_algo_signed = 0;
-                    $ns_args->{algorithm} = $algorithm;
+                    $ns_args->{algo_num} = $algorithm;
                     push @results, info( ALGO_NOT_SIGNED_RRSET => $ns_args );
                 }
             }
@@ -1979,12 +2065,14 @@ sub dnssec13 {
                     @keys = @ks;
                 }
 
+                my $msg  = q{};
+                my $time = $p->timestamp;
                 if ( not scalar @keys ) {
                     $all_algo_signed = 0;
                     $ns_args->{keytag} = $sig->keytag;
                     push @results, info( RRSIG_NOT_MATCH_DNSKEY => $ns_args );
                 }
-                elsif ( not $sig->verify( \@rrs, \@keys ) ) {
+                elsif ( not $sig->verify_time( \@rrs, \@keys, $time, $msg ) ) {
                     $all_algo_signed = 0;
                     $ns_args->{keytag} = $sig->keytag;
                     push @results, info( RRSIG_BROKEN => $ns_args );
@@ -2061,13 +2149,13 @@ sub dnssec14 {
         next if exists $investigated_keys{$key_ref};
 
         my $algo_args = {
-            algorithm_number      => $algo,
-            algorithm_description => $algo_properties{$algo}{description},
-            keytag                => $key->keytag,
-            keysize               => $key->keysize,
-            keysizemin            => $rsa_key_size_details{$algo}{min_size},
-            keysizemax            => $rsa_key_size_details{$algo}{max_size},
-            keysizerec            => $rsa_key_size_details{$algo}{rec_size},
+            algo_num   => $algo,
+            algo_descr => $algo_properties{$algo}{description},
+            keytag     => $key->keytag,
+            keysize    => $key->keysize,
+            keysizemin => $rsa_key_size_details{$algo}{min_size},
+            keysizemax => $rsa_key_size_details{$algo}{max_size},
+            keysizerec => $rsa_key_size_details{$algo}{rec_size},
         };
 
         if ( $key->keysize < $rsa_key_size_details{$algo}{min_size} ) {
